@@ -315,11 +315,16 @@ int Assemble(struct supertype *st, char *mddev,
 			}
 			/* It is worth looking inside this container.
 			 */
+			if (verbose > 0)
+				fprintf(stderr, Name ": looking in container %s\n",
+					devname);
 		next_member:
 			if (tmpdev->content)
 				content = tmpdev->content;
 			else
 				content = tst->ss->container_content(tst);
+			if (!content)
+				goto loop; /* empty container */
 
 			tmpdev->content = content->next;
 			if (tmpdev->content == NULL)
@@ -403,6 +408,9 @@ int Assemble(struct supertype *st, char *mddev,
 					fprintf(stderr, Name ": member %s in %s is already assembled\n",
 						content->text_version,
 						devname);
+			skip:
+				if (tmpdev->content)
+					goto next_member;
 				tst->ss->free_super(tst);
 				tst = NULL;
 				content = NULL;
@@ -410,6 +418,21 @@ int Assemble(struct supertype *st, char *mddev,
 					goto loop;
 				return 1;
 			}
+			if (ident->member && ident->member[0]) {
+				char *s = strchr(content->text_version+1, '/');
+				if (s == NULL) {
+					fprintf(stderr, Name ": badly formatted version: %s\n",
+						content->text_version);
+					goto skip;
+				}
+				if (strcmp(ident->member, s+1) != 0) {
+					if (report_missmatch)
+						fprintf(stderr,
+							Name ": skipping wrong member %s\n",
+							content->text_version);
+					goto skip;
+				}
+			}
 			st = tst; tst = NULL;
 			if (!auto_assem && tmpdev->next != NULL) {
 				fprintf(stderr, Name ": %s is a container, but is not "
@@ -418,6 +441,9 @@ int Assemble(struct supertype *st, char *mddev,
 				st->ss->free_super(st);
 				return 1;
 			}
+			if (verbose > 0)
+				fprintf(stderr, Name ": found match on member %s in %s\n",
+					content->text_version, devname);
 			break;
 		}
 		if (st == NULL)
@@ -563,6 +589,7 @@ int Assemble(struct supertype *st, char *mddev,
 #endif
 	/* Ok, no bad inconsistancy, we can try updating etc */
 	bitmap_done = 0;
+	content->update_private = NULL;
 	for (tmpdev = devlist; tmpdev; tmpdev=tmpdev->next) if (tmpdev->used == 1) {
 		char *devname = tmpdev->devname;
 		struct stat stb;
@@ -715,6 +742,8 @@ int Assemble(struct supertype *st, char *mddev,
 		}
 		devcnt++;
 	}
+	free(content->update_private);
+	content->update_private = NULL;
 
 	if (devcnt == 0) {
 		fprintf(stderr, Name ": no devices found for %s\n",
@@ -961,6 +990,10 @@ int Assemble(struct supertype *st, char *mddev,
 	if (content->reshape_active) {
 		int err = 0;
 		int *fdlist = malloc(sizeof(int)* bestcnt);
+		if (verbose)
+			fprintf(stderr, Name ":%s has an active reshape - checking "
+				"if critical section needs to be restored\n",
+				chosen_name);
 		for (i=0; i<bestcnt; i++) {
 			int j = best[i];
 			if (j >= 0) {
@@ -975,13 +1008,15 @@ int Assemble(struct supertype *st, char *mddev,
 				fdlist[i] = -1;
 		}
 		if (!err)
-			err = Grow_restart(st, content, fdlist, bestcnt, backup_file);
+			err = Grow_restart(st, content, fdlist, bestcnt, backup_file, verbose);
 		while (i>0) {
 			i--;
 			if (fdlist[i]>=0) close(fdlist[i]);
 		}
 		if (err) {
 			fprintf(stderr, Name ": Failed to restore critical section for reshape, sorry.\n");
+			if (backup_file == NULL)
+				fprintf(stderr,"      Possibly you needed to specify the --backup-file\n");
 			close(mdfd);
 			return err;
 		}
@@ -1090,7 +1125,20 @@ int Assemble(struct supertype *st, char *mddev,
 			      content->array.layout, clean, avail, okcnt) &&
 		       (okcnt >= req_cnt || start_partial_ok)
 			     ))) {
-			if (ioctl(mdfd, RUN_ARRAY, NULL)==0) {
+			/* This array is good-to-go.
+			 * If a reshape is in progress then we might need to
+			 * continue monitoring it.  In that case we start
+			 * it read-only and let the grow code make it writable.
+			 */
+			int rv;
+#ifndef MDASSEMBLE
+			if (content->reshape_active &&
+			    content->delta_disks <= 0)
+				rv = Grow_continue(mdfd, st, content, backup_file);
+			else
+#endif
+				rv = ioctl(mdfd, RUN_ARRAY, NULL);
+			if (rv == 0) {
 				if (verbose >= 0) {
 					fprintf(stderr, Name ": %s has been started with %d drive%s",
 						mddev, okcnt, okcnt==1?"":"s");
