@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
+ * Copyright (C) 2001-2009 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -19,12 +19,7 @@
  *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *    Author: Neil Brown
- *    Email: <neilb@cse.unsw.edu.au>
- *    Paper: Neil Brown
- *           School of Computer Science and Engineering
- *           The University of New South Wales
- *           Sydney, 2052
- *           Australia
+ *    Email: <neilb@suse.de>
  *
  *    Added by Dale Stephenson
  *    steph@snapserver.com
@@ -34,18 +29,19 @@
 #include	"md_u.h"
 #include	"md_p.h"
 
-int Kill(char *dev, int force, int quiet)
+int Kill(char *dev, int force, int quiet, int noexcl)
 {
 	/*
 	 * Nothing fancy about Kill.  It just zeroes out a superblock
 	 * Definitely not safe.
 	 */
 
-	void *super;
 	int fd, rv = 0;
 	struct supertype *st;
-		
-	fd = open(dev, O_RDWR|O_EXCL);
+
+	if (force)
+		noexcl = 1;
+	fd = open(dev, O_RDWR|(force ? 0 : O_EXCL));
 	if (fd < 0) {
 		if (!quiet)
 			fprintf(stderr, Name ": Couldn't open %s for write - not zeroing\n",
@@ -60,15 +56,13 @@ int Kill(char *dev, int force, int quiet)
 		close(fd);
 		return 1;
 	}
-	rv = st->ss->load_super(st, fd, &super, dev);
+	rv = st->ss->load_super(st, fd, dev);
 	if (force && rv >= 2)
 		rv = 0; /* ignore bad data in superblock */
 	if (rv== 0 || (force && rv >= 2)) {
-		mdu_array_info_t info;
-		info.major_version = -1; /* zero superblock */
-		free(super);
-		st->ss->init_super(st, &super, &info, 0, "", NULL);
-		if (st->ss->store_super(st, fd, super)) {
+		st->ss->free_super(st);
+		st->ss->init_super(st, NULL, 0, "", NULL, NULL);
+		if (st->ss->store_super(st, fd)) {
 			if (!quiet)
 				fprintf(stderr, Name ": Could not zero superblock on %s\n",
 					dev);
